/*
 * NativeC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Execute Latex and dvipng on the equation latex file.
 *
 * @author Jess Thrysoee
 */
public class NativeC
{
    private static final File tmpDir = new File(System.getProperty("java.io.tmpdir"));
    
    private final PreferencesC preferences;
    private final ProcessBuilder processBuilder;
    
    private InputHandlerC inputHandler;
    
    
    /**
     * Creates a new instance of NativeC
     */
    public NativeC(PreferencesC preferences) throws IOException
    {
        this.preferences = preferences;
        this.processBuilder = new ProcessBuilder();
    }
    
    
    public PNGImageC exec(String input) throws NativeExceptionC, IOException, InterruptedException
    {
        PNGImageC pngImage = null;
        
        File latexFile = writeLatexFile(input);
        String basePath = latexFile.getPath().replaceAll("\\.tex$", "");
        
        try
        {
            if (runLatex(latexFile) == 0)
            {
                File dviFile = new File(basePath + ".dvi");
                String pngFilePath = basePath + ".png";
                
                if (!dviFile.exists())
                {
                    throw new NativeExceptionC("Dvi file not found",
                            "The current latex document does not result in a dvi file.");
                }
                
                if (runDvipng(dviFile, pngFilePath) != 0)
                {
                    throw new NativeExceptionC("Dvipng Error", inputHandler.getText());
                }
                
                pngImage = new PNGImageC(new File(pngFilePath));
            }
            else
            {
                throw new NativeExceptionC("Latex Error", inputHandler.getText());
            }
        }
        finally
        {
            deleteTempFiles(basePath);
        }
        
        return pngImage;
    }
    
    
    private File writeLatexFile(String input) throws IOException
    {
        BufferedWriter out = null;
        File latexFile = File.createTempFile("laeqed", ".tex");
        
        try
        {
            out = new BufferedWriter(new FileWriter(latexFile));
            out.write("\\nonstopmode\n");
            out.write(preferences.getLatexPreamble().trim());
            out.newLine();
            out.write(input.trim());
            out.newLine();
            out.write(preferences.getLatexPostamble().trim());
        }
        finally
        {
            if (out != null) out.close();
        }
        
        return latexFile;
    }
    
    
    private int runLatex(File latexFile) throws IOException, InterruptedException
    {
        List<String> cmd = new ArrayList<String>();
        cmd.add(preferences.getLatexApplication());
        cmd.add(latexFile.getPath());
        
        return runProcessBuilder(cmd);
    }
    
    
    private int runDvipng(File dviFile, String pngFilePath) throws IOException, InterruptedException
    {
        List<String> cmd = new ArrayList<String>();
        cmd.add(preferences.getDvipngApplication());
        cmd.addAll(preferences.getDvipngOptionsList());
        cmd.add("-o");
        cmd.add(pngFilePath);
        cmd.add(dviFile.getPath());
        
        return runProcessBuilder(cmd);
    }
    
    
    private int runProcessBuilder(List<String> command) throws IOException, InterruptedException
    {
        processBuilder.command(command);
        processBuilder.directory(tmpDir);
        processBuilder.redirectErrorStream(true);
        
        Process process = processBuilder.start();
        inputHandler = new InputHandlerC(process.getInputStream());
        
        inputHandler.start();
        int exitValue = process.waitFor();
        inputHandler.join(3000);   // wait for all input to be read
        
        return exitValue;
    }
    
    
    private void deleteTempFiles(String basePath)
    {
        String[] extensions = {".tex", ".log", ".dvi", ".aux", ".png"};
        
        for (int i = 0; i < extensions.length; i++)
            (new File(basePath + extensions[i])).delete();
    }
    
    
    private static class InputHandlerC extends Thread
    {
        private InputStream is;
        private StringBuffer text;
        
        private IOException ioex;
        
        InputHandlerC(InputStream is)
        {
            this.is = is;
            this.text = new StringBuffer();
        }
        
        public String getText() throws IOException
        {
            synchronized(text)
            {
                if (ioex != null)
                {
                    throw ioex;
                }
                
                return text.toString();
            }
        }
        
        public void run()
        {
            BufferedReader buf = new BufferedReader(new InputStreamReader(is));
            String line;
            try
            {
                try
                {
                    synchronized(text)
                    {
                        while ((line = buf.readLine()) != null)
                        {
                            text.append(line).append("\n");
                        }
                    }
                }
                finally
                {
                    buf.close();
                }
            }
            catch (IOException ex)
            {
                ioex = ex;
            }
        }
    }
    
}

