/*
 * PNGImageC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed;

import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import javax.imageio.IIOException;
import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageWriter;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;

/**
 *
 * @author Jess Thrysoee
 */
public class PNGImageC
{
    private static final String LAEQED_PREAMBLE = "LaeqedPreamble";
    private static final String LAEQED_POSTAMBLE = "LaeqedPostamble";
    private static final String LAEQED_DVIPNGOPTIONS = "LaeqedDvipngOptions";
    private static final String LAEQED_EQUATION = "LaeqedEquation";
    
    private IIOImage mImage;
    private IIOMetadata mMetadata;
    
    private Node mRootNode;
    private Node mTEXtNode;
    
    private File mTempFile;
    
    
    /** Creates a new instance of PNGMetaDataC */
    public PNGImageC(File file) throws IOException
    {
        this(new BufferedInputStream(new FileInputStream(file)));
    }
    
    /** Creates a new instance of PNGMetaDataC */
    public PNGImageC(InputStream is) throws IOException
    {
        this(ImageIO.createImageInputStream(is));
    }
    
    /** Creates a new instance of PNGMetaDataC */
    public PNGImageC(ImageInputStream iin) throws IOException
    {
        
        ImageReader reader = null;
        Iterator readers = ImageIO.getImageReadersByFormatName("png");
        
        if (readers.hasNext())
        {
            try
            {
                reader = (ImageReader)readers.next();
                
                reader.setInput(iin);
                
                mImage = reader.readAll(0, null);
                
                mMetadata = mImage.getMetadata();
                mRootNode = mMetadata.getAsTree(mMetadata.getNativeMetadataFormatName());
                // debug
                //printMetadata(rootNode);
                
                mTEXtNode = gettEXt(mRootNode);
            }
            
            finally
            {
                if (iin != null) iin.close();
                if (reader != null) reader.dispose();
            }
        }
    }
    
    
    public void writeTo(File file) throws IOException
    {
        writeTo(ImageIO.createImageOutputStream(file));
    }
    
    public void writeTo(ImageOutputStream iout) throws IOException
    {
        ImageWriter writer = null;
        Iterator writers = ImageIO.getImageWritersByFormatName("png");
        
        if (writers.hasNext())
        {
            try
            {
                writer = (ImageWriter)writers.next();
                writer.setOutput(iout);
                
                mMetadata.setFromTree(mMetadata.getNativeMetadataFormatName(), mRootNode);
                
                // debug
                //printMetadata(iioMetadata.getAsTree(iioMetadata.getNativeMetadataFormatName()));
                
                writer.write(mImage);
            }
            finally
            {
                if (iout != null) iout.close();
                if (writer != null) writer.dispose();
            }
        }
    }
    
    
    /**
     * Get the image from this. Use for display.
     */
    public BufferedImage getBufferedImage()
    {
        return (BufferedImage) mImage.getRenderedImage();
    }
    
    
    /**
     * Get PNGImageC as a byte array
     */
    public byte[] getBytes() throws IOException
    {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ImageOutputStream ios = ImageIO.createImageOutputStream(bos);
        writeTo(ios);
        return bos.toByteArray();
    }
    
    
    /**
     * Get PNGImageC as a File.
     * 
     * Call removeFile() to remove stale cashed file, e.g. in settEXtEntry()
     */
    public File getFile() throws IOException
    {
        if (mTempFile == null || !mTempFile.exists())
        {
            mTempFile = File.createTempFile("laeqed", ".png");
            writeTo(mTempFile);
        }
        
        return mTempFile;
    }

    
    // metadata accessors/mutators
    public void setLatexPreamble(String s)
    {
        settEXtEntry(LAEQED_PREAMBLE, s);
    }
    public String getLatexPreamble() throws IIOException
    {
        return gettEXtEntry(LAEQED_PREAMBLE);
    }
    
    public void setLatexPostamble(String s)
    {
        settEXtEntry(LAEQED_POSTAMBLE, s);
    }
    public String getLatexPostamble() throws IIOException
    {
        return gettEXtEntry(LAEQED_POSTAMBLE);
    }
    
    public void setDvipngOptions(String s)
    {
        settEXtEntry(LAEQED_DVIPNGOPTIONS, s);
    }
    public String getDvipngOptions() throws IIOException
    {
        return gettEXtEntry(LAEQED_DVIPNGOPTIONS);
    }
    
    public void setLatexEquation(String s)
    {
        settEXtEntry(LAEQED_EQUATION, s);
    }
    public String getLatexEquation() throws IIOException
    {
        return gettEXtEntry(LAEQED_EQUATION);
    }
    
    
    private void settEXtEntry(String keyword, String value)
    {
        final String tEXtEntry = "tEXtEntry";
        
        boolean updated = false;
        Node tEXtEntryNode;
        
        NodeList nodeList = mTEXtNode.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++)
        {
            if (tEXtEntry.equals(nodeList.item(i).getNodeName()))
            {
                tEXtEntryNode = nodeList.item(i);
                
                NamedNodeMap map = tEXtEntryNode.getAttributes();
                Node keywordNode = map.getNamedItem("keyword");
                
                if (keyword.equals(keywordNode.getNodeValue()))
                {
                    map.getNamedItem("value").setNodeValue(value);
                    updated = true;
                    break;
                }
            }
        }
        
        if (!updated)
        {
            IIOMetadataNode newtEXtEntryNode = new IIOMetadataNode(tEXtEntry);
            newtEXtEntryNode.setAttribute("keyword", keyword);
            newtEXtEntryNode.setAttribute("value", value);
            mTEXtNode.appendChild(newtEXtEntryNode);
        }
        
        
        removeTempFile();   // cashed file is now stale.
    }
    
    
    private String gettEXtEntry(String keyword) throws IIOException
    {
        final String tEXtEntry = "tEXtEntry";
        Node tEXtEntryNode;
        String tEXtEntryValue = null;
        
        NodeList nodeList = mTEXtNode.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++)
        {
            if (tEXtEntry.equals(nodeList.item(i).getNodeName()))
            {
                tEXtEntryNode = nodeList.item(i);
                
                NamedNodeMap map = tEXtEntryNode.getAttributes();
                Node keywordNode = map.getNamedItem("keyword");
                
                if (keyword.equals(keywordNode.getNodeValue()))
                {
                    tEXtEntryValue = map.getNamedItem("value").getNodeValue();
                    break;
                }
            }
        }
        
        if (tEXtEntryValue == null)
        {
            throw new IIOException("tEXtEntry with keyword '"+keyword+"' not found!");
        }
        
        return tEXtEntryValue;
    }
    
    
    private Node gettEXt(Node rootNode)
    {
        final String tEXt = "tEXt";
        Node tEXtNode = null;
        
        NodeList nodeList = rootNode.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++)
        {
            if (tEXt.equals(nodeList.item(i).getNodeName()))
            {
                tEXtNode = nodeList.item(i);
                break;
            }
        }
        if (tEXtNode == null)
        {
            tEXtNode = new IIOMetadataNode(tEXt);
            rootNode.appendChild(tEXtNode);
        }
        
        return tEXtNode;
    }
    
        
    /**
     * Call when cashed mTempFile is stale, e.g. in settEXtEntry().
     */
    private void removeTempFile()
    {
        if (mTempFile != null)
        {
            mTempFile.delete();
        }
        mTempFile = null;
    }

    
    private void printMetadata(Node node)
    {
        try
        {
            DOMImplementationRegistry registry = DOMImplementationRegistry.newInstance();
            DOMImplementationLS impl = (DOMImplementationLS) registry.getDOMImplementation("LS");
            
            LSSerializer writer = impl.createLSSerializer();
            LSOutput output = impl.createLSOutput();
            writer.getDomConfig().setParameter("format-pretty-print", Boolean.TRUE);
            output.setByteStream(System.out);
            writer.write(node, output);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }
    
}
