/*
 * PreferencesC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */


package dk.thrysoee.laeqed;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.prefs.Preferences;

/**
 * @author Jess Thrysoee
 */
public class PreferencesC
{
    
    private final static String newline = System.getProperty("line.separator");
    
    private final Preferences prefs;
    private final PropertyChangeSupport propertyChange = new PropertyChangeSupport(this);
    
    // prefs keys
    private static final String LATEX_APP = "latexApp";
    private static final String LATEX_PREAMBLE = "latexPreamble";
    private static final String LATEX_POSTAMBLE = "latexPostamble";
    private static final String DVIPNG_APP = "dvipngApp";
    private static final String DVIPNG_OPTIONS = "dvipngOptions";
    private static final String SHOW_COMMAND_DETAILS = "showCommandDetails";
    
    // prefs default values
    private static final String LATEX_APP_DEFAULT = "latex";
    
    private static final String LATEX_PREAMBLE_DEFAULT =
            "\\documentclass{article}" + newline +
            "\\usepackage{amssymb,amsmath}" + newline +
            "\\usepackage{mathptmx}% Times Roman font" + newline +
            "\\pagestyle{empty}" + newline +
            "\\begin{document}" + newline +
            "\\begin{displaymath}";
    
    private static final String LATEX_EXAMPLE_EQUATION =
            "\\text{d}s^2 = (c\\;\\text{d}t^2) - a^2(t) \\left[ \\frac{\\text{d}r^2}{1 - kr^2} + " +
            "r^2 \\left( \\text{d}\\theta^2 + \\sin^2 \\theta \\text{d}\\phi^2 \\right) \\right]";
    
    private static final String LATEX_POSTAMBLE_DEFAULT =
            "\\end{displaymath}" + newline +
            "\\newpage" + newline +
            "\\end{document}" + newline;
    
    private static final String DVIPNG_APP_DEFAULT = "dvipng";
    
    private static final String DVIPNG_OPTIONS_DEFAULT = "-T tight -x 1200 -z 9 -bg Transparent";
    
    private static final boolean SHOW_COMMAND_DETAILS_DEFAULT = false;
    
    
    public PreferencesC()
    {
        prefs = Preferences.userNodeForPackage(this.getClass());
    }
    
    
    public String getLatexApplicationDefault()
    {
        return LATEX_APP_DEFAULT;
    }
    
    public synchronized String getLatexApplication()
    {
        return prefs.get(LATEX_APP, LATEX_APP_DEFAULT);
    }
    
    public synchronized void setLatexApplication(String latexApplication)
    {
        prefs.put(LATEX_APP, latexApplication);
    }
    
    public String getLatexPreambleDefault()
    {
        return LATEX_PREAMBLE_DEFAULT;
    }
    
    public synchronized String getLatexPreamble()
    {
        return prefs.get(LATEX_PREAMBLE, LATEX_PREAMBLE_DEFAULT);
    }
    
    public synchronized void setLatexPreamble(String latexPreamble)
    {
        setLatexPreamble(latexPreamble, true);
    }
    
    public synchronized void setLatexPreamble(String latexPreamble, boolean firePropertyChange)
    {
        String oldLatexPreamble = getLatexPreamble();
        prefs.put(LATEX_PREAMBLE, latexPreamble);
        if (firePropertyChange)
        {
            propertyChange.firePropertyChange("LatexPreamble", oldLatexPreamble, latexPreamble);
        }
    }
    
    public String getLatexPostambleDefault()
    {
        return LATEX_POSTAMBLE_DEFAULT;
    }
    
    public synchronized String getLatexPostamble()
    {
        return prefs.get(LATEX_POSTAMBLE, LATEX_POSTAMBLE_DEFAULT);
    }
    
    public synchronized void setLatexPostamble(String latexPostamble)
    {
        setLatexPostamble(latexPostamble, true);
    }
    
    public synchronized void setLatexPostamble(String latexPostamble, boolean firePropertyChange)
    {
        String oldLatexPostamble = getLatexPostamble();
        prefs.put(LATEX_POSTAMBLE, latexPostamble);
        if (firePropertyChange)
        {
            propertyChange.firePropertyChange("LatexPostamble", oldLatexPostamble, latexPostamble);
        }
    }
    
    public String getLatexExampleEquation()
    {
        return LATEX_EXAMPLE_EQUATION;
    }
    
    public String getDvipngApplicationDefault()
    {
        return DVIPNG_APP_DEFAULT;
    }
    
    public synchronized String getDvipngApplication()
    {
        return prefs.get(DVIPNG_APP, DVIPNG_APP_DEFAULT);
    }
    
    public synchronized void setDvipngApplication(String dvipngApplication)
    {
        prefs.put(DVIPNG_APP, dvipngApplication);
    }
    
    public String getDvipngOptionsDefault()
    {
        return DVIPNG_OPTIONS_DEFAULT;
    }
    
    public synchronized String getDvipngOptions()
    {
        return prefs.get(DVIPNG_OPTIONS, DVIPNG_OPTIONS_DEFAULT);
    }
    
    public synchronized List<String> getDvipngOptionsList()
    {
        String[] opts = prefs.get(DVIPNG_OPTIONS, DVIPNG_OPTIONS_DEFAULT).split("\\s+");
        return Arrays.asList(opts);
    }
    
    public synchronized void setDvipngOptions(String dvipngOptions)
    {
        setDvipngOptions(dvipngOptions, true);
    }
    
    public synchronized void setDvipngOptions(String dvipngOptions, boolean firePropertyChange)
    {
        String oldDvingOptions = getDvipngOptions();
        prefs.put(DVIPNG_OPTIONS, dvipngOptions);
        if (firePropertyChange)
        {
            propertyChange.firePropertyChange("DvipngOptions", oldDvingOptions, dvipngOptions);
        }
    }
    
    public synchronized boolean getShowCommandDetails()
    {
        return prefs.getBoolean(SHOW_COMMAND_DETAILS, SHOW_COMMAND_DETAILS_DEFAULT);
    }
    
    public synchronized void setShowCommandDetails(boolean isSelected)
    {
        prefs.putBoolean(SHOW_COMMAND_DETAILS, isSelected);
    }
    
    public synchronized void setMruFileList(List<File> fileList)
    {
        int i = fileList.size();
        for (File f : fileList)
        {
            prefs.put("MruFile" + Integer.toString(i--), f.getPath());
        }
        prefs.putInt("MruFileCount", fileList.size());
    }
    
    public synchronized List<File> getMruFileList()
    {
        List<File> fileList = new ArrayList<File>();
        
        int i = prefs.getInt("MruFileCount", -1);
        if (i != -1)
        {
            while (i > 0)
            {
                String path = prefs.get("MruFile" + Integer.toString(i--), null);
                if (path != null)
                {
                    fileList.add(new File(path));
                }
            }
        }
        return fileList;
    }
    
    public synchronized void addPropertyChangeListener(PropertyChangeListener l)
    {
        propertyChange.addPropertyChangeListener(l);
    }
    
    public synchronized void addPropertyChangeListener(String propertyName, PropertyChangeListener l)
    {
        propertyChange.addPropertyChangeListener(propertyName, l);
    }
    
    public synchronized void removePropertyChangeListener(PropertyChangeListener l)
    {
        propertyChange.removePropertyChangeListener(l);
    }
    
    public synchronized void removePropertyChangeListener(String propertyName, PropertyChangeListener l)
    {
        propertyChange.removePropertyChangeListener(propertyName, l);
    }
    
}

