/*
 * MathSymbolActionC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed.latexeditor.symbolsmenu;

import dk.thrysoee.laeqed.latexeditor.*;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.text.BadLocationException;

/**
 *
 * @author Jess Thrysoee
 */
public class MathSymbolActionC extends AbstractAction
{
    private static final Logger log = Logger.getLogger(MathSymbolActionC.class.getName());
    
    private final LatexEditorPaneC editorPane;
    private final String latexCommand;
    public ResizableIcon icon;
    
    
    /**
     * Creates a new instance of MathSymbolActionC
     */
    public MathSymbolActionC(LatexEditorPaneC editorPane, String label, String latexCommand, String iconResource)
    {
        super(label);
        
        this.editorPane = editorPane;
        this.latexCommand = latexCommand;
        this.icon = new ResizableIcon(iconResource);
        
        putValue(Action.SMALL_ICON, icon);
    }
    
    
    /**
     * Update editor with <code>latexCommand</code> when invoked.
     *
     * @param e
     */
    public void actionPerformed(ActionEvent e)
    {
        try
        {
            editorPane.getDocument().insertString(editorPane.getCaretPosition(), latexCommand, null);
        }
        catch (BadLocationException ex)
        {
            ex.printStackTrace();
        }
    }
    
    
    /**
     * Icon which is meant to be resized to the max width of its siblings in the JMenu.
     * This is so the menu item can be aligned even if their icons have different widths.
     */
    public static class ResizableIcon extends ImageIcon
    {
        private Image image;
        private int width;
        private int height;
        
        /**
         *
         */
        public ResizableIcon(String iconResource)
        {
            if (iconResource != null)
            {
                loadImage(iconResource);
            }
        }
        
        
        @Override
        public void paintIcon(Component c, Graphics g, int x_ignored, int y)
        {
            if (image != null)
            {
                // center image
                int x = (width - image.getWidth(null))/2 + 4;
                //y = (height - image.getHeight(null))/2;
                // Paint a PNG having translucency information.
                g.drawImage(image, x, y, null);
            }
        }
        
        
        /**
         * Get icon from resource.
         */
        public void loadImage(String resource)
        {
            InputStream in = getClass().getClassLoader().getResourceAsStream(resource);
            if (in == null)
            {
                log.warning(resource +" not found.");
            }
            else
            {
                try
                {
                    try
                    {
                        byte[] buffer = new byte[in.available()];
                        in.read(buffer);
                        image = Toolkit.getDefaultToolkit().createImage(buffer);
                        
                        setImage(image);
                        
                        width = image.getWidth(null);
                        height = image.getHeight(null);
                    }
                    finally
                    {
                        in.close();
                    }
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }
            }
        }
        
        
        /**
         * Gets the width of the icon.
         *
         * @return the width in pixels of this icon
         */
        public void setIconWidth(int width)
        {
            this.width = width;
        }
        
        
        /**
         * Gets the width of the icon.
         *
         * @return the width in pixels of this icon
         */
        public int getIconWidth()
        {
            return width;
        }
        
        
        /**
         * Gets the height of the icon.
         *
         * @return the height in pixels of this icon
         */
        public int getIconHeight()
        {
            return height;
        }
        
        
        /**
         * Gets the height of the icon.
         *
         * @return the height of the icon
         */
        public int getAccessibleIconHeight()
        {
            return height;
        }
        
        
        /**
         * Gets the width of the icon.
         *
         * @return the width of the icon
         */
        public int getAccessibleIconWidth()
        {
            return width;
        }
    }
    
}
