/*
 * MostRecentlyUsedFileCache.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.utils.mru;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.event.EventListenerList;


/**
 * Most-recently-used (MRU) file menu.
 *
 * @author Jess Thrysoee
 */
public class MruMenuC extends JMenu
{
    private final LRUMap lruMap;
    private final EventListenerList eventListenerList;
    private final int maxLabelLength;
    
    /**
     * Creates a new instance of MruEventC
     *
     * @param maxEntries number of entries in the MRU list. If maxEntries > 9 no mnemonics
     * are added to the JMenuItem's
     */
    public MruMenuC(int maxEntries, int maxLabelLength, List<File> fileList)
    {
        this.maxLabelLength = maxLabelLength;
        
        lruMap = new LRUMap(maxEntries);
        setText("Open Recent");
        
        eventListenerList = new EventListenerList();
        
        for (File f : fileList)
        {
            lruMap.put(f);
        }
        updateMenuItems();
    }
    
    
    /**
     * Add a new JMenuItem representing the <code>file</code>.
     * @param key
     */
    public void add(File file)
    {
        lruMap.put(file);
        updateMenuItems();
    }
    
    
    public void remove(File file)
    {
        if (lruMap.remove(file) != null)
        {
            updateMenuItems();
        }
    }
    
    
    public List<File> getFileList()
    {
        return lruMap.getFileList();
    }
    
    
    /**
     * Update the menu with new JMenuItems or rearrange the order.
     */
    public final void updateMenuItems()
    {
        removeAll();
        
        int i = 0;
        int size = lruMap.size();
        
        for (Map.Entry<File, JMenuItem> e : lruMap.entrySet())
        {
            File file = e.getKey();
            JMenuItem item = e.getValue();
            
            if (size < 10)
            {
                item.setText(Integer.toString(size - i) + " " + pruneLabel(file.getPath()));
                item.setMnemonic((byte)(size - i + '0'));
            }
            else
            {
                item.setText(pruneLabel(file.getPath()));
            }
            
            insert(item, 0);
            i++;
        }
    }
    
    
    /**
     * Prune the JMenuItem label if its length exceeds <code>maxLabelLength</code>
     */
    private String pruneLabel(String label)
    {
        String separator = File.separator;
        
        StringBuilder prefix = new StringBuilder();
        StringBuilder postfix = new StringBuilder();
        
        String[] split = label.split("\\" + separator);
        
        int i = 0;
        int j = split.length - 1;
        int curLength = 0;
        
        for (int k = 0; k < split.length; k++)
        {
            if (i == j) separator = "";
            
            if (k%2 == 0)
            {
                if (k != 0 && curLength + split[j].length() > maxLabelLength)
                {
                    postfix.insert(0, "...");
                    break;
                }
                else
                {
                    postfix.insert(0, separator + split[j]);
                }
                j--;
            }
            else
            {
                if (curLength + split[i].length() > maxLabelLength)
                {
                    prefix.append("...");
                    break;
                }
                else
                {
                    prefix.append(split[i] + separator);
                }
                i++;
            }
            curLength = prefix.length() + postfix.length();
        }
        
        return prefix.toString() + postfix.toString();
    }
    
    
    /**
     * Add a listener of MruEventC's
     * @param l the interested listener
     */
    public void addMruListener(MruListenerI l)
    {
        eventListenerList.add(MruListenerI.class, l);
    }
    
    
    /**
     * Remove a listener of MruEventC's
     * @param l
     */
    public void removeMruListenerC(MruListenerI l)
    {
        eventListenerList.remove(MruListenerI.class, l);
    }
    
    
    /**
     * Notify all listeners that have registered interest for
     * notification on this event type.
     *
     * @param file the file associated with the JMenuItem event
     */
    protected void fireMruActionPerformed(File file)
    {
        // Guaranteed to return a non-null array
        Object[] listeners = eventListenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length-2; i>=0; i-=2)
        {
            if (listeners[i]==MruListenerI.class)
            {
                MruEventC mruEvent = new MruEventC(this, file);
                ((MruListenerI)listeners[i+1]).mruActionPerformed(mruEvent);
            }
        }
    }
    
    
    /**
     * Least recently used map. The model of the class.
     */
    private class LRUMap extends LinkedHashMap<File, JMenuItem>
    {
        private static final float LOAD_FACTOR = 0.75f;
        private final int maxEntries;
        
        public LRUMap(int maxEntries)
        {
            super(maxEntries, LOAD_FACTOR, true);
            this.maxEntries = maxEntries;
        }
        
        public boolean removeEldestEntry(Map.Entry eldest)
        {
            return size() > maxEntries;
        }
        
        public JMenuItem put(File key)
        {
            if (containsKey(key))
            {
                return super.get(key);   // touch to make it mru
            }
            else
            {
                JMenuItem item = new JMenuItem();
                
                item.addActionListener(new ActionListener()
                {
                    public void actionPerformed(ActionEvent evt)
                    {
                        File file = get((JMenuItem) evt.getSource());
                        fireMruActionPerformed(file);
                        updateMenuItems();
                    }
                });
                
                return super.put(key, item);
            }
        }
        
        public File get(JMenuItem item)
        {
            for (Map.Entry<File, JMenuItem> e : entrySet())
            {
                if (item == e.getValue())
                {
                    super.get(e.getKey());   // touch to make it mru
                    return e.getKey();
                }
                
            }
            return null;
        }
        
        public List<File> getFileList()
        {
            List<File> fileList = new ArrayList<File>(maxEntries);
            for (File f : keySet())
            {
                fileList.add(f);
            }
            return fileList;
        }
        
        @Override
        public JMenuItem put(File key, JMenuItem value)
        {
            throw new UnsupportedOperationException("Use the put(File key) method instead");
        }
        
        @Override
        public void putAll(Map<? extends File, ? extends JMenuItem> t)
        {
            throw new UnsupportedOperationException("Use the put(File key) method instead");
        }
    }
    
}
